@@ -23,10 +23,19 @@
 #include "enums.h"
 #include "desktopicon.h"
 #include "settings.h"
+#include "formlayoutattached.h"
+#include "mnemonicattached.h"
 
 #include <QQmlEngine>
 #include <QQmlContext>
 #include <QQuickItem>
+#include <QQuickStyle>
+
+#ifdef KIRIGAMI_BUILD_TYPE_STATIC
+#include "libkirigami/platformtheme.h"
+#else
+#include <platformtheme.h>
+#endif
 
 static QString s_selectedStyle;
 
@@ -45,16 +54,16 @@ QUrl KirigamiPlugin::componentUrl(const QString &fileName) const
 void KirigamiPlugin::registerTypes(const char *uri)
 {
     Q_ASSERT(uri == QLatin1String("org.kde.kirigami"));
-    const QString style = QString::fromLatin1(qgetenv("QT_QUICK_CONTROLS_STYLE"));
+    const QString style = QQuickStyle::name();
 
     //org.kde.desktop.plasma is a couple of files that fall back to desktop by purpose
     if ((style.isEmpty() || style == QStringLiteral("org.kde.desktop.plasma")) && QFile::exists(resolveFilePath(QStringLiteral("/styles/org.kde.desktop")))) {
 #if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
         m_stylesFallbackChain.prepend(QStringLiteral("org.kde.desktop"));
 #elif defined(Q_OS_ANDROID)
-        m_stylesFallbackChain.prepend(QStringLiteral("material"));
+        m_stylesFallbackChain.prepend(QStringLiteral("Material"));
 #else // do we have an iOS specific style?
-        m_stylesFallbackChain.prepend(QStringLiteral("material"));
+        m_stylesFallbackChain.prepend(QStringLiteral("Material"));
 #endif
     }
 
@@ -64,13 +73,13 @@ void KirigamiPlugin::registerTypes(const char *uri)
         if (style == QStringLiteral("org.kde.desktop") && QFile::exists(resolveFilePath(QStringLiteral("/styles/org.kde.desktop.plasma")))) {
             m_stylesFallbackChain.prepend("org.kde.desktop.plasma");
         }
+    } else {
+        m_stylesFallbackChain.prepend(QStringLiteral("org.kde.desktop"));
     }
     //At this point the fallback chain will be selected->org.kde.desktop->Fallback
 
-
-    //TODO: in this plugin it will end up something similar to
-    //PlasmaCore's ColorScope?
     s_selectedStyle = m_stylesFallbackChain.first();
+
     qmlRegisterSingletonType<Settings>(uri, 2, 0, "Settings",
          [](QQmlEngine*, QJSEngine*) -> QObject* {
              Settings *settings = new Settings;
@@ -81,7 +90,9 @@ void KirigamiPlugin::registerTypes(const char *uri)
 
     qmlRegisterUncreatableType<ApplicationHeaderStyle>(uri, 2, 0, "ApplicationHeaderStyle", "Cannot create objects of type ApplicationHeaderStyle");
 
+    //old legacy retrocompatible Theme
     qmlRegisterSingletonType(componentUrl(QStringLiteral("Theme.qml")), uri, 2, 0, "Theme");
+
     qmlRegisterSingletonType(componentUrl(QStringLiteral("Units.qml")), uri, 2, 0, "Units");
 
     qmlRegisterType(componentUrl(QStringLiteral("Action.qml")), uri, 2, 0, "Action");
@@ -101,7 +112,12 @@ void KirigamiPlugin::registerTypes(const char *uri)
 
     //The icon is "special: we have to use a wrapper class to QIcon on org.kde.desktops
 #if !defined(Q_OS_ANDROID) && !defined(Q_OS_IOS)
-    qmlRegisterType<DesktopIcon>(uri, 2, 0, "Icon");
+    //we know that we want a different implementation with Material and Plasma styles
+    if (s_selectedStyle == QStringLiteral("Material") || s_selectedStyle == QStringLiteral("Plasma")) {
+        qmlRegisterType(componentUrl(QStringLiteral("Icon.qml")), uri, 2, 0, "Icon");
+    } else {
+        qmlRegisterType<DesktopIcon>(uri, 2, 0, "Icon");
+    }
 #else
     qmlRegisterType(componentUrl(QStringLiteral("Icon.qml")), uri, 2, 0, "Icon");
 #endif
@@ -121,6 +137,15 @@ void KirigamiPlugin::registerTypes(const char *uri)
     qmlRegisterType(componentUrl(QStringLiteral("AbstractApplicationItem.qml")), uri, 2, 1, "AbstractApplicationItem");
     qmlRegisterType(componentUrl(QStringLiteral("ApplicationItem.qml")), uri, 2, 1, "ApplicationItem");
 
+    //2.2
+    //Theme changed from a singleton to an attached property
+    qmlRegisterUncreatableType<Kirigami::PlatformTheme>(uri, 2, 2, "Theme", "Cannot create objects of type Theme, use it as an attached poperty");
+
+    //2.3
+    qmlRegisterType(componentUrl(QStringLiteral("FormLayout.qml")), uri, 2, 3, "FormLayout");
+    qmlRegisterUncreatableType<FormLayoutAttached>(uri, 2, 3, "FormData", "Cannot create objects of type FormData, use it as an attached poperty");
+    qmlRegisterUncreatableType<MnemonicAttached>(uri, 2, 3, "MnemonicData", "Cannot create objects of type MnemonicData, use it as an attached poperty");
+
     qmlProtectModule(uri, 2);
 }
 
